<?php

declare(strict_types=1);

namespace Drupal\commerce_coinspaid\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_coinspaid\Service\CoinspaidService;
use Drupal\commerce_coinspaid\Service\DebugService;
use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_payment\Annotation\CommercePaymentGateway;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Exception;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the CoinsPaid offsite Checkout payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "coinspaid_redirect",
 *   label = @Translation("CoinsPaid (Redirect to payment page)"),
 *   display_label = @Translation("CoinsPaid"),
 *    forms = {
 *     "offsite-payment" =
 *   "Drupal\commerce_coinspaid\PluginForm\OffsiteRedirect\CoinspaidOffsiteForm",
 *   },
 * )
 */
class OffsiteRedirect extends OffsitePaymentGatewayBase
{
  /**
   * {@inheritdoc}
   */
    public function defaultConfiguration(): array
    {
        $route_name = \Drupal::routeMatch()->getRouteName();
        $params = \Drupal::routeMatch()->getParameters();
        $request = \Drupal::request();
        $is_ajax = $request->isXmlHttpRequest();

        if (
            !$is_ajax &&
            !empty($this->configuration['mode']) && $this->configuration['mode'] === 'test' &&
            $route_name === 'commerce_checkout.form' &&
            ($params->has('step') && $params->get('step') === 'order_information')
        ) {
            \Drupal::messenger()->addMessage(
                t('Pay by crypto. You are using a sandbox payment method.'),
                MessengerInterface::TYPE_WARNING
            );
        }
        return [
        'public_key' => '',
        'secret_key' => '',
        'invoice_mode' => 0,
        'debug' => false,
        ] + parent::defaultConfiguration();
    }

  /**
   * {@inheritdoc}
   */
    public function buildConfigurationForm(
        array $form,
        FormStateInterface $form_state
    ): array {
        $form = parent::buildConfigurationForm($form, $form_state);

        $form['public_key'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Public Key'),
        '#description' => $this->t('This is the Public Key from the CoinsPaid portal.'),
        '#default_value' => $this->configuration['public_key'],
        '#required' => true,
        ];

        $form['secret_key'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Secret key'),
        '#description' => $this->t('This is the Secret key from the CoinsPaid portal.'),
        '#default_value' => $this->configuration['secret_key'],
        '#required' => true,
        ];

        $form['invoice_mode'] = [
        '#type' => 'select',
        '#title' => $this->t('Invoice Mode'),
        '#description' => $this->t('The invoice mode for the payment form.'),
        '#options' => $this->getInvoiceModes(),
        '#default_value' => $this->configuration['invoice_mode'],
        ];
        $form['debug'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Debug Mode'),
        '#description' => $this->t('Enable debug mode.'),
        '#default_value' => $this->configuration['debug'],
        ];

        return $form;
    }

  /**
   * Returns an array of invoice modes supported by CoinsPaid.
   *
   * @return array
   */
    private function getInvoiceModes(): array
    {
        return [
        $this->t('Invoice without restriction of payment time'),
        $this->t('Invoice with time restriction'),
        ];
    }

  /**
   * {@inheritdoc}
   */
    public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void
    {
        parent::submitConfigurationForm($form, $form_state);
        if (!$form_state->getErrors()) {
            $values = $form_state->getValue($form['#parents']);
            $this->configuration['public_key'] = $values['public_key'];
            $this->configuration['secret_key'] = $values['secret_key'];
            $this->configuration['invoice_mode'] = $values['invoice_mode'];
            $this->configuration['debug'] = $values['debug'];
        }
    }

  /**
   * @param Request $request
   */
    public function onNotify(Request $request): void
    {
        $headers = $request->headers->all();
        $data_json = $request->getContent();
        $response_data = json_decode($data_json, true);
        DebugService::setEnabled((bool) $this->configuration['debug']);
        DebugService::log($response_data, 'Callback - responseData');
        DebugService::log($headers, 'Callback - headers');
        if (!empty($response_data['foreign_id'])) {
            try {
                $order_id = $this->getOrderId($response_data['foreign_id']);
                $order = Order::load($order_id);
                $coinspaid_service = new CoinspaidService(
                    $this->configuration['public_key'],
                    $this->configuration['secret_key'],
                    $this->configuration['mode']
                );
                $coinspaid_service->validateCallback($response_data, $headers);
                $coinspaid_service->parseResponse($response_data);

                if ($coinspaid_service->getResponseStatus() === CoinspaidService::STATUS_ERROR) {
                      $order->set('state', 'cancelled');
                      $order->save();
                } elseif ($coinspaid_service->getResponseStatus() === CoinspaidService::STATUS_SUCCESS) {
                    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');

                    $last = $payment_storage->loadByProperties([
                    'payment_gateway' => $this->parentEntity->id(),
                    'order_id' => $order_id,
                    'remote_id' => $coinspaid_service->getTransactionId(),
                    ]);
                    if (!empty($last)) {
                          $payment_storage->delete($last);
                    }
                    $payment = $payment_storage->create([
                    'state' => 'completed',
                    'amount' => $order->getTotalPrice(),
                    'payment_gateway' => $this->parentEntity->id(),
                    'order_id' => $order_id,
                    'remote_id' => $coinspaid_service->getTransactionId(),
                    'remote_state' => $coinspaid_service->getResponseStatus(),
                    ]);
                    $payment->save();
                    die('Ok');
                }
            } catch (Exception $exception) {
                \Drupal::logger('commerce_coinspaid')->error($exception->getMessage());
            }
        } else {
            \Drupal::logger('commerce_coinspaid')->error('foreign_id is empty');
        }
    }

  /**
   * Parse order id
   *
   * @param string $foreign_id
   *
   * @return string
   */
    private function getOrderId(string $foreign_id): string
    {
        $foreign_array = explode('-', $foreign_id);
        array_pop($foreign_array);

        return implode('-', $foreign_array);
    }
}
