<?php

declare(strict_types=1);

namespace Drupal\commerce_coinspaid\Service;

use Exception;

/**
 *  Class CoinspaidService
 */
class CoinspaidService
{
    public const STATUS_SUCCESS = 'confirmed';

    public const STATUS_PENDING = 'pending';

    public const STATUS_PROCESSING = 'processing';

    public const STATUS_ERROR = 'error';

    private array $server = [
    'test' => 'https://app.sandbox.cryptoprocessing.com/api/v2',
    'live' => 'https://es.cryptoprocessing.com/api/v2/',
    ];

    private array $headersKey = [
    'public' => 'x-processing-key',
    'signature' => 'x-processing-signature',
    ];

    private string $env = '';

    private string $publicKey = '';

    private string $secretKey = '';

    private array $errors = [];

    private ?array $response = null;

    private string $orderId;

    private string $responseStatus = '';

    private int $responseCode;

    private string $transactionId;

  /**
   * Create a new CoinspaidService instance.
   *
   * @param string $public_key
   * @param string $secret_key
   * @param string $env
   */
    public function __construct(string $public_key, string $secret_key, string $env = 'live')
    {
        if (!empty($public_key) && !empty($secret_key)) {
            $this->publicKey = $public_key;
            $this->secretKey = $secret_key;
        }
        if (!empty($env) && (($env == 'live') || ($env == 'test'))) {
            $this->env = $env;
        }
    }

  /**
   * Ping
   *
   * @return null|array
   * @throws Exception
   */
    public function ping(): ?array
    {
        $command = '/ping';
        $result = $this->execute('GET', $command, [], true);

        if (!empty($result['data'])) {
            return $result;
        } else {
            $this->errors[] = $result;

            return null;
        }
    }

  /**
   * Get Supported Currencies
   *
   * @return null|array
   * @throws Exception
   */
    public function getSupportedCurrencies(): ?array
    {
        $command = '/currencies/list';
        $result = $this->execute('POST', $command, [], true);

        if (!empty($result['data'])) {
            return $result['data'];
        } else {
            $this->errors[] = $result;

            return null;
        }
    }

  /**
   * Create invoice
   *
   * @param array $params
   *
   * @return null|array
   * @throws Exception
   */
    public function createInvoice(array $params): ?array
    {
        $command = '/invoices/create';

        $result = $this->execute('POST', $command, $params, true);
        DebugService::log($result, 'createInvoice - response');
        if (!empty($result['data'])) {
            return $result;
        } elseif (is_array($result)) {
            $this->errors = $result;
            return null;
        } else {
            $this->errors[] = $result;
            return null;
        }
    }

  /**
   * Execute command
   *
   * @param string $method
   * @param string $command
   * @param array $params
   * @param bool $json
   *
   * @return array|null
   * @throws Exception
   */
    private function execute(string $method, string $command, array $params = [], bool $json = false): ?array
    {
        $this->errors = [];

        if ($method && $command) {
            $curl_options = [
            CURLOPT_URL => $this->server[$this->env] . $command,
            CURLOPT_HEADER => true,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_INFILESIZE => null,
            CURLOPT_HTTPHEADER => [],
            CURLOPT_CONNECTTIMEOUT => 10,
            CURLOPT_TIMEOUT => 10,
            ];

            $curl_options[CURLOPT_HTTPHEADER][] = 'Accept-Charset: utf-8';
            $curl_options[CURLOPT_HTTPHEADER][] = 'Accept: application/json';
            $curl_options[CURLOPT_HTTPHEADER][] = 'Content-Type: application/json';
            $curl_options[CURLOPT_HTTPHEADER][] = 'X-Processing-Key: ' . $this->publicKey;
            $curl_options[CURLOPT_HTTPHEADER][] = 'X-Processing-Signature: ' . $this->generateSignature($params);
            try {
                switch (strtolower(trim($method))) {
                    case 'get':
                        $curl_options[CURLOPT_HTTPGET] = true;
                        $curl_options[CURLOPT_URL] .= '?' . $this->buildQuery($params, $json);
                        break;
                    case 'post':
                        $curl_options[CURLOPT_POST] = true;
                        $curl_options[CURLOPT_POSTFIELDS] = $this->buildQuery($params, $json);
                        break;
                    case 'head':
                        $curl_options[CURLOPT_NOBODY] = true;
                        break;
                    default:
                        $curl_options[CURLOPT_CUSTOMREQUEST] = strtoupper($method);
                }
            } catch (Exception $exception) {
                \Drupal::logger('commerce_coinspaid')->error($exception->getMessage());
            }

            $ch = curl_init();
            curl_setopt_array($ch, $curl_options);
            $response = curl_exec($ch);

            if (curl_errno($ch)) {
                $curl_code = curl_errno($ch);
                $constant = get_defined_constants(true);
                $curl_constant = preg_grep('/^CURLE_/', array_flip($constant['curl']));

              /*DebugService::log([
                'name' => $curl_constant[$curl_code],
                'message' => curl_strerror($curl_code),
              ],
                'execute: ' . $method);*/
            }

            $status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            $this->responseCode = $status_code;
            if (($status_code >= 0) && ($status_code < 200)) {
                DebugService::log('Server Not Found (' . $status_code . ')', 'execute: ' . $method);
            }

            if (($status_code >= 300) && ($status_code < 400)) {
                DebugService::log('Page Redirect (' . $status_code . ')', 'execute: ' . $method);
            }

            if (($status_code >= 400) && ($status_code < 500)) {
                DebugService::log('Page not found (' . $status_code . ')', 'execute: ' . $method);
            }

            if ($status_code >= 500) {
                DebugService::log('Server Error (' . $status_code . ')', 'execute: ' . $method);
            }

            $head = '';
            $body = '';
            $parts = [];
            if ($response) {
                $parts = explode("\r\n\r\n", $response, 3);
            }
            if (isset($parts[0]) && isset($parts[1])) {
                if (($parts[0] == 'HTTP/1.1 100 Continue') && isset($parts[2])) {
                    [$head, $body] = [$parts[1], $parts[2]];
                } else {
                    [$head, $body] = [$parts[0], $parts[1]];
                }
            }

            $response_headers = [];
            $header_lines = explode("\r\n", $head);
            array_shift($header_lines);

            foreach ($header_lines as $line) {
                [$key, $value] = explode(':', $line, 2);
                $response_headers[$key] = $value;
            }

            curl_close($ch);

            $response_array = json_decode($body, true);
            if (!empty($response_array['errors'])) {
                $response_array = $response_array['errors'];
            }
            $this->response = $response_array;

            return $response_array;
        }

        return null;
    }

  /**
   * Generate signature
   *
   * @param array $params_array
   *
   * @return string
   * @throws Exception
   */
    public function generateSignature(array $params_array): string
    {
        $request_body = json_encode($params_array);
        if ($request_body) {
            return hash_hmac('sha512', $request_body, $this->secretKey);
        } else {
            return throw new Exception('GenerateSignature is failed. Params invalid');
        }
    }

  /**
   * Build query
   *
   * @param array $params
   * @param bool $json
   *
   * @return string
   * @throws Exception
   */
    private function buildQuery(array $params, bool $json): string
    {
        if ($json) {
            $json_params = json_encode($params);
            if ($json_params) {
                return $json_params;
            } else {
                return throw new Exception('Invalid query params');
            }
        } else {
            return http_build_query($params);
        }
    }

  /**
   * Has errors
   *
   * @return bool
   */
    public function hasErrors(): bool
    {
        return (bool) count($this->errors);
    }

  /**
   * Get errors data
   *
   * @return array
   */
    public function getErrors(): array
    {
        return $this->errors;
    }

  /**
   * Get str errors data
   *
   * @return string
   */
    public function getErrorsStr(): string
    {
        return json_encode($this->errors) ?? '';
    }

  /**
   * Get redirect link
   *
   * @return string
   */
    public function getRedirectLink(): string
    {
        if (!empty($this->response['data']['url'])) {
            return $this->response['data']['url'];
        }
        return '';
    }

  /**
   * Get response data
   *
   * @return array
   */
    public function getResponseData(): array
    {
        if (!empty($this->response)) {
            return $this->response;
        }
        return [];
    }

  /**
   * Validate callback
   *
   * @param array $headers
   * @param array $data
   *
   * @return bool
   * @throws Exception
   * @throws Exception
   */
    public function validateCallback(array $data, array $headers): bool
    {
        $signature = $this->generateSignature($data);

        if (!empty($headers[$this->headersKey['public']]) && !empty($headers[$this->headersKey['signature']])) {
            $callback_public = array_shift($headers[$this->headersKey['public']]);
            $callback_signature = array_shift($headers[$this->headersKey['signature']]);
            if ($callback_public === $this->publicKey && $callback_signature === $signature) {
                return true;
            }
        }
        return throw new Exception('Signature failed validation');
    }

  /**
   * Parse callback params from response
   *
   * @param array $data
   *
   * @return bool
   */
    public function parseResponse(array $data): bool
    {
        $valid = true;
        if (!isset($data['foreign_id'])) {
            \Drupal::logger('commerce_coinspaid')->error('foreign_id doesn\'t exist');
            $valid = false;
        }
        if (!isset($data['status'])) {
            \Drupal::logger('commerce_coinspaid')->error('status doesn\'t exist');
            $valid = false;
        }
        if (!$valid) {
            return false;
        }
        $this->orderId = $this->parseOrderId($data['foreign_id']);
        $this->responseStatus = $data['status'];
        $this->transactionId = $this->parseTransactionId($data);

        return true;
    }

  /**
   * Parse order id from foreign Id
   *
   * @param string $foreignId
   *
   * @return string
   */
    private function parseOrderId(string $foreignId): string
    {
        $foreignArray = explode('-', $foreignId);
        array_pop($foreignArray);
        return implode('-', $foreignArray);
    }

  /**
   * Parse transaction id from response
   *
   * @param array $data
   *
   * @return string
   */
    private function parseTransactionId(array $data): string
    {
        $transaction_id = $data['id'] ?? '';
        if (isset($data['transactions']) && is_array($data['transactions'])) {
            foreach ($data['transactions'] as $transaction) {
                if (isset($transaction['txid'])) {
                    $transaction_id = $transaction['txid'];
                }
            }
        }

        return (string) $transaction_id;
    }

  /**
   * Get order id
   *
   * @return string
   */
    public function getOrderId(): string
    {
        return $this->orderId;
    }

  /**
   * Get response status
   *
   * @return string
   */
    public function getResponseStatus(): string
    {
        return $this->responseStatus;
    }

  /**
   * Get response code
   *
   * @return int
   */
    public function getResponseCode(): int
    {
        return $this->responseCode;
    }

  /**
   * Get transaction id
   *
   * @return string
   */
    public function getTransactionId(): string
    {
        return $this->transactionId;
    }
}
